/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "outputbuild.h"

OutputBuild::OutputBuild()
{
  swin = gtk_scrolled_window_new(NULL, NULL);
  build_box = gtk_tree_view_new();
  
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(build_box), false);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), build_box);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
}

GtkWidget* OutputBuild::get_build_box()
{ return build_box;
}

GtkWidget* OutputBuild::get_swin()
{ return swin;
}

// Run the specified command with pipes. This function receives the command to
// run, the TreeView object and the environment settings class object. It 
// cleans the build output and runs the command.
//
// This function has the ability to highlight warnings, errors and other compiler
// messages with the user-specified colors as well as check if the build is able
// to find Makefiles
bool OutputBuild::run_command (string command, 
                               EnvironmentSettings *env)
{
  char buff[32];
  FILE *in;
  bool successful = true, autogenerate = false;
  int location;
  string text = "\0", line;
  GtkTreeIter iter;
  
  erase();
  add_row(command.substr(0, command.find("2>&1", 0)), "#000000");
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(build_box));
  gtk_tree_model_get_iter(model, &iter, gtk_tree_path_new_from_string("0"));
  gtk_tree_selection_select_iter(selection, &iter);
    
  // Open pipe & loop until the program is complete
  in = popen(command.c_str(), "r");
  
  while (fgets(buff, sizeof(buff), in) && continue_build) 
  {
    text += buff;
    // Find the endl's & split them to be added to build_box
    location = text.find("\n",0);
    while (location != (signed int)string::npos) 
    {
      // Check to see if autogen.sh must be run before build
      if (text.find("No targets specified",0) != string::npos)
        autogenerate = true;
        
      // Append line & erase it from the buffer
      line = text.substr(0,location);
      text.erase(0,location+1);
      location = text.find("\n",0);

      // Apply Error or Warning colors
      if (line.find("error: ") != string::npos) 
      {
        add_row(line, env->get_error_color());
        successful = false;
      } 
      else if (line.find(": ") != string::npos)
        add_row(line, env->get_warning_color());
      else
        add_row(line, "#000000");
      
      // Update the build_box when there is output
      update_output();
    }
  }
    
  // Close the stream, & print whether command was successful or not
  pclose(in);
  if (successful)
    add_row("Completed ... Successful", "#000000");
  else
    add_row("Completed ... Failed", "#000000");
  update_output();
  
  // Return false if the user must run autogen.sh before building & then do it
  if (autogenerate)
    return false;
  
  return true;
}

void OutputBuild::erase()
{
  // Clean up build_box & get it ready to accept output
  if (gtk_tree_view_get_model(GTK_TREE_VIEW(build_box)))
        gtk_tree_view_remove_column(GTK_TREE_VIEW(build_box),
            gtk_tree_view_get_column(GTK_TREE_VIEW(build_box), 0));                            
  model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, GDK_TYPE_COLOR));
  gtk_tree_view_set_model(GTK_TREE_VIEW(build_box), model);
  
  // Append the columns & create the cell renderer
  renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Search Results", 
           renderer, "text", 0, "foreground-gdk", 1, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(build_box), column);

  continue_build = true;
}

void OutputBuild::add_row (string text, 
                           string color)
{
  GtkTreeIter iter;
  GdkColor tmp = { 0, 0, 0, 0, },
           *gcolor = gdk_color_copy(&tmp);
  
  // Add the command to the box
  gdk_color_parse(color.c_str(), gcolor);
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
                     0, text.c_str(), 
                     1, gcolor, -1);
}

void OutputBuild::update_output()
{
  GtkTreeIter iter;
  
  gtk_tree_model_get_iter_first (model, &iter);
  while (TRUE)
  {
    GtkTreeIter next = iter;
    if (gtk_tree_model_iter_next (model, &next))
      iter = next;
    else
      break;
  }
  
  while(gtk_events_pending()) 
  {
    gtk_main_iteration();
    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(build_box), 
                                 gtk_tree_model_get_path(model, &iter), 0, 
                                 TRUE, 0.0, 0.0);
  }
}

void OutputBuild::stop_build()
{ continue_build = false;
}
